# When is ESC Calibration Required?
- when a new type of motor or new type of propeller is used
- when switching battery types (from 3S LiPo to 4S LiPo, etc)

# Hardware Setup
- Battery (check the voltage requirements for the specific ESC). Please note that testing with a power supply can damage the ESC, see [regenerative braking](doc/regenerative_braking.md)
- use appropriate wire gauge for the maximum current (usually 18 or 16 gauge is OK) and try to minimize the length of the power cable
- identify the motor that will be used for tuning and attach the propeller to that motor. This calibration must be done with propeller attached!
- make sure the frame with the motor is firmly attached to some platform and cannot move when thrust is applied
- connect power connector from battery to ESC
- connect UART adapter to PC / Laptop
- apply power and make sure there are no unusual sounds or smell (besides the short beep ESC makes with motors when power is applied)

# Software Setup
- download voxl-esc tools and install any required prerequisites
 - see instructions at https://gitlab.com/voxl-public/voxl-sdk/utilities/voxl-esc/-/tree/master/voxl-esc-tools
- locate an existing tuning params file that most closely matches the motor that you have
- make a copy of the existing params file and update it if needed
    - specifically, make sure that ```num_cycles_per_rev``` is correct (usually 6 or 7 corresponding to number of pole pairs), otherwise RPM will not be calculated correctly
    - set ```vbat_nomival_mv``` to nominal battery voltage
    - leave other parameters the same, as they may not be known
    - upload the params file ```./voxl-esc-upload-params.py --params-file ../voxl-esc-params/<params_file>.xml```

# Detect ESC and Perform a Quick Spin Test
- detect the ESCs, make sure the software is the same version
 - ```./voxl-esc-scan.py```
- do a quick spin at lower power (10%)
 - ```./voxl-esc-spin.py --id 0 --power 10```
 - make sure the propeller is spinning in the correct direction (CW or CCW depending on propeller). Reverse one pair of motor wires to switch direction if needed
 - compare RPM to what is reported by tachometer
 - do not command very large power (open loop commands), since it will result in a very large step (less than 30)

## Calibrate ESC
- wear safety glasses
  - make sure the propeller is clear of any obstacles and there are no items that can become airborne due to high air flow
- calibrate the ESC's feed-forward curve
 - ```./voxl-esc-calibrate.py --id 0 --pwm-min 10 --pwm-max 95 --pwm-step 3 --pwm-step-duration 0.7```
  - this will step the motor through a range of power from 10% to 95%


## Update and Upload Calibration File
- update calibration parameters file printed by the calibration script
 - ```pwm_vs_rpm_curve_a0, pwm_vs_rpm_curve_a1, pwm_vs_rpm_curve_a2```
 - ```min_rpm``` and ```max_rpm```. Note that max rpm will depend on battery voltage, but it is a good practice to just use maximum RPM at nominal voltage (or lowest voltage). minimum rpm can be taked from 10% power and maximum from 95% power. However, do not command 95% power using voxl-esc-spin.py, since there is no limiting in power mode (as opposed to RPM) and motor can burn out.
- upload the calibration file to all ESCs. This will also reboot all the ESCs.
 - ```./voxl-esc-upload-params.py --params-file ../voxl-esc-params/<params_file>.xml```
- validate the esc parameters: ```./voxl-esc-verify-params.py```

## Test ESC using RPM commands
- choose RPM to test according to motor limits
- with proper tuning, the motor should track RPM very closely (typically within 100 RPM across the whole range)
- voltage compensation is automatic, so try adjusting voltage up and down 1-2 volts and RPM tracking should remain constant
```
./voxl-esc-spin.py --id 0 --rpm 5000
...
[0] RPM: 4999, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.79C, CURRENT: 0.18A
[0] RPM: 4999, PWR: 20, VBAT: 11.45V, TEMPERATURE: 31.80C, CURRENT: 0.22A
[0] RPM: 5001, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.82C, CURRENT: 0.14A
[0] RPM: 5001, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.82C, CURRENT: 0.14A
[0] RPM: 4979, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.76C, CURRENT: 0.21A
[0] RPM: 5005, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.67C, CURRENT: 0.22A
[0] RPM: 5011, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.64C, CURRENT: 0.16A
[0] RPM: 5011, PWR: 20, VBAT: 11.44V, TEMPERATURE: 31.64C, CURRENT: 0.16A
...
```
